#!/bin/env python
#DoEgen: A Python Library for Optimised Design of Experiment Generation and Evaluation
#
#Copyright 2020 Sebastian Haan
# 
#DoEgen is free software made available under the AGPL License. 
#For details see the LICENSE file.
#
#@author: Sebastian Haan

"""
DoEgen is a Python library aiming to assist in generating optimised
Design of Experiments (DoE), evaluating design efficiencies, and
analysing experiment results.

In a first step, optimised designs can be automatically generated and
efficiencies evaluated for any mixture of factor-levels for numeric and
categorical factors. Designs are automatically evaluated as function of
number of experiment runs and the most efficient designs are suggested.
In particular DoEgen provides computation of a wide range of design
efficiencies and allows to import and evaluate externally generated
designs as well.

The second part of DoEgen assists in analysing any derived experiment
results in terms of factor importance, correlations, and response
analysis for best parameter space selection.

Definitions
-----------

An Experiment Design is typically defined by:

-   Number of Factors: the parameters or variates of the experiment
-   Number of Runs: the number of experiments
-   Levels: The number of value options for each factor, which can be
    either numeric values (discrete or continuous) or categorical.
    Discrete levels for continuous factors can be obtained by providing
    the minimum and maximum of the factor range and the number of
    levels. The more levels, the more fine-grained the experiment will
    evaluate this factor, but also more experimental runs are required.

The goal of optimising an experimental design is to provide an efficient
design that is near-optimal in terms of, e.g., orthogonality, level
balance, and two-way interaction coverage, yet can be performed with a
minimum number of experimental runs, which are often costly or
time-consuming.

Functionality
-------------

If you would like to jumpstart a new experiment and to skip the
technical details, you can find a summary of the main usage of DoEgen in
Case Study Use Case in the README.

Currently, the (preliminary) release contains several functions for
generating and evaluating designs. Importing and evaluating external
designs is supported (e.g. for comparison to other DoE generator tools).
DoE also implements several functions for experiment result analysis and
visualisation of parameter space.

The main functionalities are (sorted in order of typical experiment
process):

-   Reading Experiment Setup Table and Settings (Parameter Name, Levels
    for each factor, Maximum number of runs, Min/Max etc)
-   Generating optimised design arrays for a range of runs (given
    maximum number of runs, and optional computation-time constrains,
    see `settings_design.yaml`).
-   Evaluation and visualisation of more than ten design efficiencies
    such as level balance, orthogonality, D-efficiencies etc (see
    [Design Efficiencies](#design-efficiencies) for the complete list).
-   Automatic suggestion of minimum, optimal, and best designs within a
    given range of experiment runs.
-   Import and evaluation of externally generated design arrays.
-   Experiment result analysis: Template table for experiment results,
    multi-variant RMSE computation, best model/parameter selection,
    Factor Importance computation, pairwise response surface and
    correlation computation, factor correlation analysis and Two-way
    interaction response plots.
-   Visualisation of experiment results.

Installation And Requirements
-----------------------------

### Requirements

-   Python >= 3.6
-   OApackage
-   xlrd
-   XlsxWriter
-   Numpy
-   Pandas
-   PyYAML
-   scikit_learn
-   matplotlib
-   seaborn

The DoEgen package is currently considered experimental and has been
tested with the libraries specified in `requirements.txt`.

Installation instructions and documentation for OApackage (tested with
OApackage 2.6.6) can be found at https://pypi.org/project/OApackage/ or
can be installed with

pip install OAPackage

Please see for more details the README.
"""

__version__ = "0.1"
__author__ = "Sebastian Haan"


"""
Run this initialisation of .yaml and .xlsx files after installation of doegen.

Creates config yaml files and excel templates if not already existing in current working directory.
"""
import os
import sys
import shutil
import doegen

#Current working directory
cwd = os.getcwd()

#Directory where package doegen is installed:
pckd = doegen.__path__[0] 

# Create settings_design.yaml
if os.path.isfile('settings_design.yaml'):
	print('File settings_design.yaml already exists.')
else:
	source = os.path.join(pckd,'settings_design.yaml')
	target = os.path.join(cwd,'settings_design.yaml')
	try:
	    shutil.copy(source, target)
	    print('Please edit settings_design.yaml')
	except IOError as e:
	    print("Unable to copy file. %s" % e)
	except:
	    print("Unexpected error:", sys.exc_info())

# Create Experiment_setup.xlsx
if os.path.isfile('Experiment_setup.xlsx'):
	print('File Experiment_setup.xlsx already exist.')
else:
	source = os.path.join(pckd,'Experiment_setup.xlsx')
	target = os.path.join(cwd,'Experiment_setup.xlsx')
	try:
	    shutil.copy(source, target)
	    print('Please add your experiment settings in Experiment_setup.xlsx')
	except IOError as e:
	    print("Unable to copy file. %s" % e)
	except:
	    print("Unexpected error:", sys.exc_info())


# Create settings_expresults.yaml
if os.path.isfile('settings_expresults.yaml'):
	print('File settings_expresults.yaml already exists.')
else:
	source = os.path.join(pckd,'settings_expresults.yaml')
	target = os.path.join(cwd,'settings_expresults.yaml')
	try:
	    shutil.copy(source, target)
	    print('Please edit settings_expresults.yaml after running the experiment.')
	except IOError as e:
	    print("Unable to copy file. %s" % e)
	except:
	    print("Unexpected error:", sys.exc_info())

# Create Experiment_results.xlsx
if os.path.isfile('Experiment_results.xlsx'):
	print('File Experiment_results.xlsx already exist.')
else:
	source = os.path.join(pckd,'Experiment_results.xlsx')
	target = os.path.join(cwd,'Experiment_results.xlsx')
	try:
	    shutil.copy(source, target)
	    print('Please add your experiment results in Experiment_results.xlsx after running the experiment.')
	except IOError as e:
	    print("Unable to copy file. %s" % e)
	except:
	    print("Unexpected error:", sys.exc_info())


"""
Package to evaluate the response and factor effectiveness of experiment results.

Author: Sebastian Haan
Affiliation: Sydney Informatics Hub (SIH), THe University of Sydney
Version: Experimental
License: APGL-3.0

Tested with Python 3.7

Main Capabilities:
- Multi-variant RMSE computation and best model/parameter selection
- Factor Importance computation
- Pairwise response surface and correlation computation
- Factor correlation analysis and Two -way intearction response plots
- Visualisation plots

ToDo:
Change to Pathlib

Changes to previous version:
- replace configloader with function arguments
"""

import os
import sys
import argparse
import yaml
from pathlib import Path
import numpy as np
import pandas as pd
from mpl_toolkits import mplot3d
import matplotlib.pyplot as plt
import seaborn as sns

sns.set()
# Load settings parameters


def merge_expresults(fname_result, fname_design, y_label=None):
    """
	Reads experiment results into pandas dataframe 
	and merges with paramater file

	INPUT
	fname_result: path + filenmae of experimental results (see excel template)
	fname_design: path + filenmae of experimental design setip (see excel template)
	y_label: (Default None) Column name for precited y property
	"""
    dfres = pd.read_excel(fname_result)
    dfdes = pd.read_excel(fname_design)
    if y_label is not None:
        dfres = dfres[dfres["Y Label"] == y_label]
    # Merge two files:
    dfcomb = dfres.merge(dfdes, on="Nexp", how="left")
    return dfcomb


def create_testdata(outpath, fname_out, Nexp):
    """
	# Script for creating random set of results for testing
	
	INPUT
	outpath: output directory
	fname_out: filename in format '*.xlsx'
	Nexp: NUmber of experiments
	"""
    os.makedirs(outpath, exist_ok=True)
    PID = np.arange(1, 11)
    Yexp = np.random.rand(Nexp, len(PID)).flatten()
    Ytruth = np.random.rand(Nexp, len(PID)).flatten()
    Ylabel = np.ones(len(Yexp))
    aNexp = np.zeros_like(Yexp)
    aPID = np.zeros_like(Yexp)
    for i in range(Nexp):
        aNexp[10 * i : 10 * i + 10] = i + 1
        aPID[10 * i : 10 * i + 10] = PID
    array = np.vstack(
        (
            aNexp,
            aPID,
            Ylabel,
            Yexp,
            Ytruth,
            Ytruth * np.nan,
            Ytruth * np.nan,
            Ytruth * np.nan,
        )
    )
    header = [
        "Nexp",
        "PID",
        "Y Label",
        "Y Exp",
        "Y Truth",
        "Std Y Exp",
        "Std Y Truth",
        "Weight PID",
    ]
    df = pd.DataFrame(array.T, columns=header)
    df.to_excel(os.path.join(outpath,fname_out), index=False)


def weighted_avg_and_std(values, weights):
    """
    Returns weighted average and standard deviation.

    INPUT
    values, weights -- arrays with the same shape.
    """
    average = np.average(values, weights=weights)
    # Fast and numerically precise:
    variance = np.average((values - average) ** 2, weights=weights)
    return (average, np.sqrt(variance))


def calc_expresults_stats(ylabels, dfdes, dfres, outpath):
    """
	Computation of statistical evaluation of experimetal results for each predicted y:
	1) Parameter importance, which is defined by maximum y range over parameter levels (y in averaged for each level)
	Results are visualized in bar plot and saved as csv, including, min, max, std devioation across all levels
	2) Computes RMSE and saves results as csv
	3) Computes list of top experiments and their parameters
	4) Computes average and variance of best parameters weighted with RMSE; saved to csv file

	INPUT
	ylabels: label ID for each target variable.
	dfdes: experiment design dataframe (inlcudes one column Nexp and the other columns the factor names)
	dfres: experiment result dataframe (Ids in Nexp column must match design array)
    outpath: path for output files
	"""
    npar = len(list(dfdes)) - 1
    nexp = len(dfdes)
    params = list(dfdes)[1:]
    for ylabel in ylabels:
        dfdes_y = dfdes.copy()
        # Initialise array for factor results
        ymin_par = np.full(npar, np.nan)
        ymax_par = np.zeros(npar) * np.nan
        ystd_par = np.zeros(npar) * np.nan
        ymean_par = np.zeros(npar) * np.nan
        # Select Y data and to dfdes overall stats in dfdes
        ydf = dfres[dfres["Y Label"] == ylabel].copy()
        ymean = ydf.fillna(0).groupby("Nexp")["Y Exp"].mean()
        ystd = ydf.fillna(0).groupby("Nexp")["Y Exp"].std()
        ytruemean = ydf.fillna(0).groupby("Nexp")["Y Truth"].mean()
        ytruestd = ydf.fillna(0).groupby("Nexp")["Y Truth"].std()
        assert len(ymean) == dfdes_y.shape[0]
        dfdes_y["Y Exp Mean"] = ymean.values
        dfdes_y["Y Exp Std"] = ystd.values
        dfdes_y["Y Truth Mean"] = ytruemean.values
        dfdes_y["Y Truth Std"] = ytruestd.values
        # Loop over parameter to caluclate factor range, min, max, mean and stddev:
        for i, param in enumerate(params):
            levels = dfdes[param].unique()
            ylevel = [
                np.nanmean(dfdes_y.loc[dfdes[param] == level, "Y Exp Mean"])
                for level in levels
            ]
            ylevelstd = [
                np.nanmean(dfdes_y.loc[dfdes[param] == level, "Y Exp Std"])
                for level in levels
            ]
            ymin_par[i] = np.nanmin(ylevel)
            ymax_par[i] = np.nanmax(ylevel)
            ystd_par[i] = np.nanstd(ylevel)
            ymean_par[i] = np.nanmean(ylevel)
        ypos = np.arange(npar)
        width = ymax_par - ymin_par
        sort = np.argsort(width)
        # Plot factor importance as barplot
        plt.ioff()
        plt.figure(figsize=(8, 5))
        plt.barh(
            ypos,
            width=width[sort],
            left=ymin_par[sort],
            tick_label=np.asarray(params)[sort],
            color="red",
        )
        plt.title("Range " + str(ylabel))
        plt.tight_layout()
        plt.savefig(os.path.join(outpath, "Ybarplot_" + str(ylabel) + ".png"), dpi=300)
        plt.close()
        # Save factor importance to csv:
        res = np.vstack((width, ymin_par, ymax_par, ymean_par, ystd_par))
        dfrange = pd.DataFrame(
            res.T, columns=["Yrange", "Ymin", "Ymax", "Ymean", "Ystd"], index=params
        )
        dfrange.to_csv(
            os.path.join(outpath, "Experiment_" + str(ylabel) + "_Factorimportance.csv")
        )

        # Calculate RMSE and best parameter space:
        if ydf["Y Truth"].notnull().any():
            rmse = np.zeros(nexp)
            ytrue = np.zeros(nexp)
            for i in range(nexp):
                resid = (
                    ydf.loc[ydf["Nexp"] == i + 1, "Y Exp"]
                    - ydf.loc[ydf["Nexp"] == i + 1, "Y Truth"]
                )
                rmse[i] = np.sqrt(np.nanmean(resid ** 2))
            dfdes_y["RMSE"] = rmse
            # Save overall results to csv with sorted RMSE
            dfdes_y.to_csv(os.path.join(outpath, "Experiment_" + str(ylabel) + "_RMSE.csv"))

            # Calculate best parameters (for only nueric parameters)
            if nexp >= 20:
                nsel = 10
            elif (nsel >= 10) & (nsel < 20):
                nsel = 5
            else:
                nsel = 3
            dfsort = dfdes_y.sort_values(["RMSE"], ascending=True)
            print(
                "Top "
                + str(nsel)
                + " experiments with best RMSE for "
                + str(ylabel)
                + " :"
            )
            print(dfsort.head(nsel))
            dfsort.iloc[0:nsel].to_csv(
                os.path.join(outpath,
                "Experiment_"
                + str(ylabel)
                + "_RMSE_Top"
                + str(nsel)
                + "_sorted.csv")
            )
            """ takingh out best parameter weighting since averaging might be misleading
            # best parameter space is based on weighted RMSE of top results
            # Note that these are average parameter estimaets and are not considering multi-modal distributions
            # For multi-modal see list of top resulst
            # Select only numeric parameters
            params_num = dfsort[params]._get_numeric_data().columns
            param_wmean = np.zeros(len(params_num))  # weigthed mean
            param_wstd = np.zeros(len(params_num))  # weighted std
            dfsel = dfsort.iloc[0:nsel]
            for i, param in enumerate(params_num):
                param_wmean[i], param_wstd[i] = weighted_avg_and_std(
                    dfsel[param].values, 1 / (dfsel["RMSE"].values ** 2)
                )
            params_stats = np.vstack((param_wmean, param_wstd))
            # Save to csv
            dfparam_avg = pd.DataFrame(
                params_stats.T,
                index=params_num,
                columns=["Weighted Average", "Weigted Stddev"],
            )
            dfparam_avg.to_csv(
                cfg.outpath + "Experiment_" + str(ylabel) + "_Best-Parameter-Avg.csv"
            )
            # plot dataframe table
            plot_table(dfparam_avg, cfg.outpath, "BestFactor_Avg_" + str(ylabel) + ".png")
            """
            """
            dfparam_avg.plot(kind="bar", y="Weighted Average", yerr="Weigted Stddev")
            plt.tight_layout()
            """


# Make 3d correlation plot with heatmap
# (Make 3d scatter to image plot (works only for continous))
def plot_3dmap(df, params, target_name, fname_out):
    """
	Plots Y value or RMSE as function of two differnt X variates for each pairwise combination of factors
	The plot is using a gridded heatmap which enablesto visualise also categorical factors 
	and not just numerical data

	INPUT
	df: pandas dataframe
	params: list of factor names
    target_name: 'Y Exp Mean' or 'RMSE'
	dfname_out: output path + filename for image

	OUTPUT
	Cornerplot of Y-PairwiseCorrelation Images 
	"""
    print('Plotting Y as function of pairwise covariates ...')
    nfac = len(params)
    # Check first for max and min value
    ymin0 = df[target_name].max()
    ymax0 = df[target_name].min()
    for i in range(nfac - 1):
        for j in range(i + 1, nfac):
            table = pd.pivot_table(
                df,
                values=target_name,
                index=[params[j]],
                columns=[params[i]],
                aggfunc=np.nanmean,
            )
            if np.min(table.min()) < ymin0:
                ymin0 = np.min(table.min())
            if np.max(table.max()) > ymax0:
                ymax0 = np.max(table.max())
    # Make corner plot
    # sns.set_style("whitegrid")
    plt.ioff()  # automatic disables display of figures
    # fig, axs = plt.subplots(nfac-1, nfac-1, sharex=True, sharey=True, figsize=(nfac*2, nfac*2))
    fig, axs = plt.subplots(nfac - 1, nfac - 1, figsize=(nfac * 2, nfac * 2))
    for i in range(nfac - 1):
        for j in range(i + 1, nfac):
            table = pd.pivot_table(
                df,
                values=target_name,
                index=[params[j]],
                columns=[params[i]],
                aggfunc=np.nanmean,
            )
            g = sns.heatmap(
                table,
                cmap="viridis",
                annot=False,
                ax=axs[j - 1, i],
                vmin=ymin0,
                vmax=ymax0,
                square=True,
                cbar=False,
            )
            if i > 0:
                g.set_ylabel("")
                g.set(yticklabels=[])
            if j < nfac - 1:
                g.set_xlabel("")
                g.set(xticklabels=[])
    # remove remaining plots:
    for i in range(1, nfac - 1):
        for j in range(1, i + 1):
            g = sns.heatmap(
                table * np.nan,
                cmap="viridis",
                annot=False,
                ax=axs[j - 1, i],
                vmin=ymin0,
                vmax=ymax0,
                square=True,
                cbar=False,
            )
            g.set_ylabel("")
            g.set(yticklabels=[])
            g.set_xlabel("")
            g.set(xticklabels=[])
    # Make colorbar
    g = sns.heatmap(
        table * np.nan,
        cmap="viridis",
        annot=False,
        ax=axs[0, 1],
        vmin=ymin0,
        vmax=ymax0,
        square=True,
        cbar=True,
    )
    g.set_xlabel("")
    g.set_ylabel("")
    g.set(yticklabels=[])
    g.set(xticklabels=[])
    fig.suptitle("Pair-Variate Plot with " + target_name)
    plt.savefig(fname_out, dpi=300)


def plot_3dmap_rmse(df, params,  fname_out):
    """
    Plots RMSE value as function of two differnt X variates for each pairwise combination of factors
    The plot is using a gridded heatmap which enablesto visualise also categorical factors 
    and not just numerical data

    INPUT
    df: pandas dataframe
    params: list of factor names
    target_name: 
    dfname_out: output path + filename for image

    OUTPUT
    Cornerplot of Y-PairwiseCorrelation Images 
    """
    print('Plotting RMSE as function of pairwise covariates ...')
    nfac = len(params)
    # Check first for max and min value
    ymin0 = df["RMSE"].max()
    ymax0 = df["RMSE"].min()
    for i in range(nfac - 1):
        for j in range(i + 1, nfac):
            table = pd.pivot_table(
                df,
                values="RMSE",
                index=[params[j]],
                columns=[params[i]],
                aggfunc=np.nanmean,
            )
            if np.min(table.min()) < ymin0:
                ymin0 = np.min(table.min())
            if np.max(table.max()) > ymax0:
                ymax0 = np.max(table.max())
    # Make corner plot
    # sns.set_style("whitegrid")
    plt.ioff()  # automatic disables display of figures
    # fig, axs = plt.subplots(nfac-1, nfac-1, sharex=True, sharey=True, figsize=(nfac*2, nfac*2))
    fig, axs = plt.subplots(nfac - 1, nfac - 1, figsize=(nfac * 2, nfac * 2))
    for i in range(nfac - 1):
        for j in range(i + 1, nfac):
            table = pd.pivot_table(
                df,
                values="RMSE",
                index=[params[j]],
                columns=[params[i]],
                aggfunc=np.nanmean,
            )
            g = sns.heatmap(
                table,
                cmap="viridis",
                annot=False,
                ax=axs[j - 1, i],
                vmin=ymin0,
                vmax=ymax0,
                square=True,
                cbar=False,
            )
            if i > 0:
                g.set_ylabel("")
                g.set(yticklabels=[])
            if j < nfac - 1:
                g.set_xlabel("")
                g.set(xticklabels=[])
    # remove remaining plots:
    for i in range(1, nfac - 1):
        for j in range(1, i + 1):
            g = sns.heatmap(
                table * np.nan,
                cmap="viridis",
                annot=False,
                ax=axs[j - 1, i],
                vmin=ymin0,
                vmax=ymax0,
                square=True,
                cbar=False,
            )
            g.set_ylabel("")
            g.set(yticklabels=[])
            g.set_xlabel("")
            g.set(xticklabels=[])
    # Make colorbar
    g = sns.heatmap(
        table * np.nan,
        cmap="viridis",
        annot=False,
        ax=axs[0, 1],
        vmin=ymin0,
        vmax=ymax0,
        square=True,
        cbar=True,
    )
    g.set_xlabel("")
    g.set_ylabel("")
    g.set(yticklabels=[])
    g.set(xticklabels=[])
    fig.suptitle("Pair-Variate Plot for RMSE Function")
    plt.savefig(fname_out, dpi=300)


def plot_regression(df, params, target_name, fname_out):
    """
	Creates Correlation plot with Y or RMSE for each numeric Variate
	Note that only numeric data is selected for this plot

	INPUT
	df: dataframe
	params: list of factor names
    target_name: 'Y Exp Mean' or 'RMSE'

	OUTPUT
	Image with Correlations
	"""
    # Select numeric variates:
    columns = df[params]._get_numeric_data().columns
    nfac = len(columns)
    nax1 = int(np.sqrt(nfac))
    nax2 = int(np.ceil(nfac / int(np.sqrt(nfac))))
    # fig, axs = plt.subplots(nax1, nax2, figsize=(nax1 * 3, nax2 * 3))
    plt.ioff()  # automatic disables display of figures
    fig = plt.figure(figsize=(nax1 * 5, nax2 * 4))
    for i in range(nfac):
        r = df[columns[i]].corr(df[target_name])
        plt.subplot(nax2, nax1, i + 1)
        # sns.lmplot( x = columns[0], y = 'Y Exp Mean', data = df)
        ax = sns.regplot(x=columns[i], y=target_name, data=df)
        ax.annotate("r = {:.3f}".format(r), xy=(0.1, 0.9), xycoords=ax.transAxes)
        plt.savefig(fname_out, dpi=300)

def plot_factordis(df, params, target_name, fname_out):
    """
    Creates distribution plot of Y or RMSE for each numeric Variate
    Note that only numeric data is selected for this plot

    INPUT
    df: dataframe
    params: list of factor names
    target_name: 'Y Exp Mean' or 'RMSE'

    OUTPUT
    Image with Correlations
    """
    # Select numeric variates:
    columns = df[params]._get_numeric_data().columns
    nfac = len(columns)
    nax1 = int(np.sqrt(nfac))
    nax2 = int(np.ceil(nfac / int(np.sqrt(nfac))))
    # fig, axs = plt.subplots(nax1, nax2, figsize=(nax1 * 3, nax2 * 3))
    plt.ioff()  # automatic disables display of figures
    fig = plt.figure(figsize=(nax1 * 5, nax2 * 4))
    for i in range(nfac):
        plt.subplot(nax2, nax1, i + 1)
        ax = sns.violinplot(y=df[target_name], x=df[columns[i]])
        plt.savefig(fname_out, dpi=300)

def plot_table(df_table, outpath, fname_out):
    """
    Plot Dataframe as formatted table

    INPUT
    df_table: dataframe
    outpath: output path
    fname_out: image output filename
    """
    # Format table

    #df_table.to_string(float_format=lambda x: '%.3f' % x)
    plt.ioff()
    plt.figure(linewidth=2,
           tight_layout={'pad':40},
           figsize=(7,4)
          )
    # Set colors for row and column headers
    rcolors = plt.cm.BuPu(np.full(len(df_table), 0.15))
    ccolors = plt.cm.BuPu(np.full(len(list(df_table)), 0.15))

    # Hide axes
    ax = plt.gca()
    ax.get_xaxis().set_visible(False)
    ax.get_yaxis().set_visible(False)
    table = pd.plotting.table(ax, df_table, loc = 'center', 
        rowLoc='left',
        colLoc = 'center',
        rowColours=rcolors,
        colColours=ccolors)
    table.scale(0.6, 1.3)
    table.set_fontsize(7)
    plt.box(on=None)
   # plt.tight_layout(pad = 40)
    plt.draw()
    plt.savefig(os.path.join(outpath, fname_out), dpi=300)
    plt.close()
    

def main(inpath, fname_results, fname_design, outpath = None):

    if outpath is None:
        outpath = inpath = Path(inpath)
    else:
        outpath = Path(outpath)
    os.makedirs(outpath, exist_ok = True)
    # 1) Read in experiment result data
    if fname_results.endswith('.xlsx'):
        dfres = pd.read_excel(os.path.join(inpath, fname_results))
    elif fname_results.endswith('.csv'):
        dfres = pd.read_csv(os.path.join(inpath, fname_results))
    # ['Nexp' 'PID', 'Y Label', 'Y Exp', 'Y Truth', 'Std Y Exp', 'Std Y Truth', 'Weight PID']
    # 2) Read in experiment design setup table with parameter sepcifications
    dfdes = pd.read_csv(os.path.join(inpath, fname_design))
    # dfdes = pd.read_csv('designs_Danial/' + 'designtable_Nrun36.csv' )

    # List of different predictable Y properties:
    try:
        ylabels = dfres["Y Label"].unique()
    except:
        print("No column with name 'Y Label' found in results file. Default results name 'Y1' added.")
        dfres["Y Label"] = 'Y1'
        ylabels = dfres["Y Label"].unique()
    params = list(dfdes)[1:]
    npar = len(params)
    nexp = dfdes.shape[0]

    # Calculating main stats (RMSE, parameter importance, best parameters)
    calc_expresults_stats(ylabels, dfdes, dfres, outpath)

    # Visualise correlation results for each Y predictable
    for ylabel in ylabels:
        print("Plotting correlation plots for Ylabel:" + str(ylabel) + " ...")
        dfname = os.path.join(outpath, "Experiment_" + str(ylabel) + "_RMSE.csv")
        df = pd.read_csv(dfname)
        # Plot Pairwise X correlation for Y:
        fname_out1 = (os.path.join(
            outpath, "Y-pairwise-correlation_" + str(ylabel) + ".png")
        )
        plot_3dmap(df, params, "Y Exp Mean", fname_out1)
        # Plot Pairwise X correlation for RMSE
        fname_out2 = (os.path.join(
            outpath, "RMSE-pairwise-correlation_" + str(ylabel) + ".png")
        )
        plot_3dmap(df, params, "RMSE", fname_out2)
        # Plot Main factor correlation plot with Y:
        fname_out3 = os.path.join(outpath, "Expresult_correlation_X-Y_" + str(ylabel) + ".png")
        plot_regression(df, params, 'Y Exp Mean', fname_out3)
        fname_out4 = os.path.join(outpath, "Expresult_distribution_X-RMSE_" + str(ylabel) + ".png")
        plot_factordis(df, params, 'RMSE', fname_out4)

    print("FINISHED")


def main_cli():
    ap = argparse.ArgumentParser()
    ap.add_argument('settings_path', nargs='?', default='settings_expresults.yaml')
    args = ap.parse_args()
    print(f"using settings in: {args.settings_path!r}")
    with open(args.settings_path) as f:
        cfg = yaml.safe_load(f)
    main(**cfg)


if __name__ == "__main__":
    #from doegen import configloader_results as cfg
    main_cli()


"""
Generates Excel file with format for experimental design setup

Author: Sebastian Haan
Affiliation: Sydney Informatics Hub (SIH), THe University of Sydney
Version: 0.1
License: APGL-3.0
"""

import xlsxwriter

workbook = xlsxwriter.Workbook('Experiment_results_template.xlsx')
worksheet = workbook.add_worksheet()

workbook.set_properties({
    'title':    'Experimental Design Results',
    'subject':  'Template',
    'author':   'Sebastian Haan',
    'company':  'SIH, The University of Sydney',
    'comments': 'Created with Python and XlsxWriter'
})

# Add a format for the header cells.
header_format = workbook.add_format({
    'border': 2,
    'bg_color': '#C6EFCE',
    'bold': True,
    'text_wrap': True,
    'valign': 'bottom',
    'indent': 1,
    'locked': True
})

unlocked = workbook.add_format({'locked': False})
# Enable worksheet protection
#worksheet.protect(options={'autofilter': True})
#worksheet.autofilter('A1:B8')

worksheet.protect()

header_format.set_font_size(14)

worksheet.set_default_row(20)
worksheet.set_row(0, 20)

worksheet.set_column('A:H', 15, unlocked)

# Write the header cells
# Same identifier as in setupfile, this need to match with experimemnt setup file to merge with associated parameters!
heading1 = 'Nexp' 
# Optional: ID of measurement point (e.g. spatial or temporal position),
heading2 = 'PID' 
# Optional: index of multi output-target if applicable (optional)
#(e.g. Y can be distinct properties or target values even for same PID)
heading3 = 'Y Label' 
# Experiment or simulation result for given position PID and output Y Label
heading4 = 'Y Exp'
# Optional: ground truth for given PID and output Y Label
heading5 = 'Y Truth'
# Optional:  Standard deviation (noise) of experiment result for given position PID and output Y Label
heading6 = 'Std Y Exp'
# Optional:  Standard deviation (noise) of ground truth for given position PID and output Y Label
heading7 = 'Std Y Truth'
# Optional: weight for positional measurement with PID
heading8 = 'Weight PID'

worksheet.write('A1', heading1, header_format)
worksheet.write('B1', heading2, header_format)
worksheet.write('C1', heading3, header_format)
worksheet.write('D1', heading4, header_format)
worksheet.write('E1', heading5, header_format)
worksheet.write('F1', heading6, header_format)
worksheet.write('G1', heading7, header_format)
worksheet.write('H1', heading8, header_format)

#Freeze panes
worksheet.freeze_panes(1, 0)  # Freeze the first row

workbook.close()

print('Excel Template Created')


# Load settings

import argparse
import yaml

ap = argparse.ArgumentParser()
ap.add_argument('settings_path', nargs='?', default='settings_expresults.yaml')
args = ap.parse_args()
print(f"using settings in: {args.settings_path!r}")
with open(args.settings_path) as f:
	cfg = yaml.safe_load(f)
for key in cfg:
	locals()[str(key)] = cfg[key]


"""
Generates Excel file with format for experimental design setup

Author: Sebastian Haan
Affiliation: Sydney Informatics Hub (SIH), THe University of Sydney
Version: 0.1
License: APGL-3.0
"""

import xlsxwriter

workbook = xlsxwriter.Workbook('Experiment_setup_extended.xlsx')
worksheet = workbook.add_worksheet()

workbook.set_properties({
    'title':    'Experimental Design Setup (Extended Version)',
    'subject':  'Template',
    'author':   'Sebastian Haan',
    'company':  'SIH, The University of Sydney',
    'comments': 'Created with Python and XlsxWriter'
})

# Add a format for the header cells.
header_format = workbook.add_format({
    'border': 2,
    'bg_color': '#C6EFCE',
    'bold': True,
    'text_wrap': True,
    'valign': 'bottom',
    'indent': 1,
    'locked': True
})

unlocked = workbook.add_format({'locked': False})
# Enable worksheet protection
#worksheet.protect(options={'autofilter': True})
#worksheet.autofilter('A1:B8')

worksheet.protect()

header_format.set_font_size(14)

worksheet.set_default_row(20)
worksheet.set_row(0, 30)

worksheet.set_column('A:E', None, unlocked)

# Set up layout of the worksheet.
worksheet.set_column('A:A', 40, unlocked)
worksheet.set_column('B:B', 20, unlocked)
worksheet.set_column('C:C', 20, unlocked)
worksheet.set_column('D:D', 20, unlocked)
worksheet.set_column('E:E', 20, unlocked)
worksheet.set_column('F:F', 20, unlocked)
worksheet.set_column('G:G', 50, unlocked)


# Write the header cells and some data that will be used in the examples.
heading1 = 'Parameter Name'
heading2 = 'Parameter Type'
heading3 = 'Level Number'
heading4 = 'Minimum'
heading5 = 'Maximum'
heading6 = 'Include (Y/N)'
heading7 = 'Levels'

worksheet.write('A1', heading1, header_format)
worksheet.write('B1', heading2, header_format)
worksheet.write('C1', heading3, header_format)
worksheet.write('D1', heading4, header_format)
worksheet.write('E1', heading5, header_format)
worksheet.write('F1', heading6, header_format)
worksheet.write('G1', heading7, header_format)

#worksheet.write_row('B2:B10', ['Continous', 'Discrete', 'Categorical'])
worksheet.data_validation('B2:B20', {'validate': 'list',
                                  'source':  ['Continuous', 'Discrete', 'Categorical']})
#worksheet.write_row('C2:C10', ['Integers', 2, 10])
worksheet.data_validation('C2:C20', {'validate': 'integer',
                                 'criteria': 'between',
                                 'minimum': 2,
                                 'maximum': 10})
worksheet.data_validation('F2:F20', {'validate': 'list',
                                  'source':  ['Yes', 'No']})

#Freeze panes
worksheet.freeze_panes(1, 0)  # Freeze the first row

workbook.close()


#!/bin/env python
"""
Package for creation and evaluation of experiment design for any mixed-level exp desig setup.
Designs are evaluated in regard to more than 10 design efficiency criteria.
This packlage also includes also a function (eval_extarray) that allows to import externally created designs
, e.g. with SAS tools, and to evaluate their design efficencies

Author: Sebastian Haan
Affiliation: Sydney Informatics Hub (SIH), The University of Sydney
Version: 0.1
License: APGL-3.0

Tested with Python 3.7, see requirements.txt
"""

import argparse
import os
import sys
from pathlib import Path
import time
from contextlib import redirect_stdout
from collections import namedtuple
import numpy as np
import pandas as pd
import itertools
#import tabulate
#import xlrd
import oapackage

# for oapckage installation and docs see https://oapackage.readthedocs.io/en/latest/oapackage.html
from sklearn.cross_decomposition import CCA

# from mpl_toolkits import mplot3d
import matplotlib.pyplot as plt
import seaborn as sns

sns.set(style="ticks", color_codes=True)
import yaml


def gen_highD(setup, arrayclass, nkeep=2, printopt=True, outpath=None):
    """
	(In work) Extends arrays and filter based on weighted D-efficiency
	Works computationally only for small arrays and small number of runs.
	EXPERIMENTAL. NOT DEFAULT OPTMIMZATION METHOD.
	
	INPUT
	:param setup: ExperimentalSetup
	:param arrayclass: arrayclass
	:param nkeep: Number of designs to keep at each stage
	:param printout: if True prints array stats
	RETURN
	Design array and efficiencies
	"""
    alpha = [5, 5, 15]
    if printopt:
        print(arrayclass)
    arraylist = [arrayclass.create_root()]
    options = oapackage.OAextend()
    options.setAlgorithmAuto(arrayclass)
    for extension_column in range(2, setup.number_of_factors):
        print(
            f"extend {len(arraylist)} arrays with {arraylist[0].n_columns} columns with a single column"
        )
        arraylist_extensions = oapackage.extend_arraylist(
            arraylist, arrayclass, options
        )
        if arraylist_extensions:
            # Select the best arrays based on the D-efficiency
            design_efficiencies = np.array(
                [a.Defficiencies() for a in arraylist_extensions]
            )
            design_efficiencies[~np.isfinite(design_efficiencies)] = 0.0
            # dd = np.array([a.Defficiency() for a in arraylist_extensions])
            dd = (
                np.nansum(design_efficiencies * np.asarray(alpha), axis=1)
                / np.asarray(alpha).sum()
            )
            ind = np.argsort(dd)[::-1]
            ind = ind[0:nkeep]
            selection = [arraylist_extensions[ii] for ii in ind]
            dd = dd[ind]
            if printopt:
                print(
                    "  generated %d arrays, selected %d arrays with D-efficiency %.4f to %.4f"
                    % (len(arraylist_extensions), len(ind), dd.min(), dd.max())
                )
            arraylist = selection
            success = 1
        else:
            print("Failure to find array extension")
            break
            success = 0
    # show the best array
    if success:
        Asel = selection[0]
        if printopt:
            print(
                "Generated a design in OA(%d, %d, 2^%d) with D-efficiency %.4f"
                % (Asel.n_rows, arrayclass.strength, Asel.n_columns, dd[0])
            )
            print("The array is (in transposed form):\n")
            Asel.transposed().showarraycompact()
        effs = evaluate_design2(setup, np.array(Asel), printopt=True)
        return Asel, effs
    else:
        return None, None


def test_genhighD(setup, runsize, nkeep=2, outpath=None):
    """
	For some testing of function gen_highD().
	"""
    maxfact = np.max(setup.factor_levels)
    minfact = np.min(setup.factor_levels)
    if runsize % np.lcm(minfact, maxfact) > 0:
        print(
            "Number of Experiments (Runsize) must be lowest common multiple of factor levels!"
        )
    else:
        arrayclass = oapackage.arraydata_t(
            setup.factor_levels, runsize, 2, setup.number_of_factors
        )
        Asel, effs = gen_highD(setup, arrayclass, nkeep=nkeep, outpath=outpath)
        if outpath is not None:
            os.makedirs(outpath, exist_ok=True)
            fname = (
                "Oarray_" + str(setup.factor_levels) + "_Nrun" + str(runsize) + ".csv"
            )
            np.savetxt(os.path.join(outpath,fname), np.asarray(Asel), delimiter=",", fmt="%i")
            fname = (
                "Efficiencies_"
                + str(setup.factor_levels)
                + "_Nrun"
                + str(runsize)
                + ".csv"
            )
            np.savetxt(os.path.join(outpath, fname), effs, delimiter=",")
        return Asel, effs


def calc_twofactorbalance(setup, Array):
    """ 
	Computation of two-way interaction balance of exp design array and any missing	missing two-way intearctions
	INPUT
	setup: ExperimentalSetup
	Array: experiment design array with shape (N exp runs, M factors)
	RETURN
	lb2: Two-way interaction balance level, from zero (worst) to one (optimal)
	bl2_least1: fraction of missing combinations with minimum of one occurence
	Alevelbal: (Mfactors times Mfactors) Matrix of interaction counts
	"""
    runsize, nfac = Array.shape
    # fac_levels = [len(np.unique(Array[:, col].astype(int))) for col in range(Array.shape[1])]
    colname = ["Param_" + str(i) for i in range(nfac)]
    df = pd.DataFrame(data=Array, columns=colname)
    fac_levels = df.nunique(axis=0).values
    ncount = 0
    lb2_err = 0.0
    bl2_least1_err = 0.0
    Alevelbal = np.zeros((nfac, nfac))
    for i, j in itertools.combinations(range(nfac), 2):
        ncount += 1
        nperfect = runsize / (fac_levels[i] * fac_levels[j])
        df_group = (
            df.groupby(["Param_" + str(i), "Param_" + str(j)])
            .size()
            .reset_index()
            .rename(columns={0: "count"})
        )
        # Be aware that this doesn't take into account combinations with no occurrence
        cvals = df_group["count"].values
        ilb2_err = abs(cvals - nperfect)
        ilb2_err[ilb2_err > nperfect] = nperfect
        # count combinations that do not occure
        nmiss = (fac_levels[i] * fac_levels[j]) - len(cvals)
        # Add all errors
        lb2_err += (ilb2_err.sum() + nperfect * nmiss) / runsize
        bl2_least1_err += nmiss / (fac_levels[i] * fac_levels[j])
        # print(df_group)
        # print('nmiss', nmiss)
        # print('lb2_err', lb2_err)
        # print('bl2_least1_err', bl2_least1_err)
    # Normalize balance calculation
    bl2_balance = 1 - lb2_err / ncount
    bl2_least1 = 1 - bl2_least1_err / ncount

    # Create two-level interaction balance pivot table
    dfcat = (
        pd.DataFrame(data=Array, columns=setup.factor_names)
        .astype("int")
        .astype("category")
    )
    # Alevelbal = pd.get_dummies(dfcat.set_index(setup.factor_names[0]), prefix_sep = ' _').sum(level=0)
    """
	Creates list that contains for each level in each factor the  number of interaction with other factors levels
	The aim is to check if pariwise combinations occur at least one and if the number of ouccurence is evenly spread
	"""
    Alist = []
    factors = sorted(setup.factor_names)
    for factor_name in factors:
        Alist.append(
            pd.get_dummies(dfcat.set_index(factor_name), prefix_sep=" _")
            .sum(level=0)
            .astype(int)
        )
    # print(Alist)
    # Alevelbal = pd.concat(Alist, keys=factors, sort='True')
    return bl2_balance, bl2_least1, Alist  # Alevelbal


def normalize_array(Array):
    """
	Normalize array from -1 to 1
	"""
    colmax = np.max(Array, axis=0)
    colmin = np.min(Array, axis=0)
    coldelta = colmax - colmin
    colmean = (colmax + colmin) / 2.0
    return 2 * (Array - colmean) / coldelta


def evaluate_design2(setup, Array, printopt=True, dir_out=None, plotgrid=True):
    """
	Computes multiple characteristics of a given design array, such as 
	Level Balance, Center Balance, Orthogonality, D-, D_s-, D1-, A-Efficiencies
	Most indicators have a range from 0 (worst possible) to 1 (optimal)
	Creates also correlation plots for evaluation.
	INPUT
	setup: ExperimentalSetup
	Array: experiment design array with shape (N exp runs, M factors)
	printopt: (optional) boolean, by default output results 
	path_out: (optional) output dirtectoy name for diagnostis tables
	plotgrid: (optional) plots pairwise correlation of design as cornerplot
	RETURN
	list of 11 efficiencies (normalized from 0:worst to 1:optimal):
	Center Balance
	Level Balance
	Orthogonality
	Two-way Interaction Balance
	Two-way Interaction with at least one occurence
	D Efficiency (main term and quadratic)
	D1 Eff (only main terms)
	D2 Eff (main, quadratic, and interaction terms)
	A-Eff  (main term and quadratic)
	A1-Eff (only main terms)
	A2-Eff (main, quadratic, and interaction terms)
	If dir_out is not None: 
	Tables of corraltions, level balance and two-main interaction balance
	Plot of pairwise relationships
	"""
    runsize, number_of_factors = Array.shape
    # fac_levels = [len(np.unique(Array[:, col].astype(int))) for col in range(Array.shape[1])]
    fac_levels = [len(np.unique(col.astype(int))) for col in Array.T]
    # Normalize Array
    Anorm = normalize_array(Array)
    # Create Model Matrix X and Eff
    X, _ = create_model(Anorm, mode=2, norm=False)
    Deff = calc_Deff(X)
    Aeff = calc_Aeff(X)
    X1, _ = create_model(Anorm, mode=1, norm=False)
    D1eff = calc_Deff(X1)
    A1eff = calc_Aeff(X1)
    X2, _ = create_model(Anorm, mode=4, norm=False)
    D2eff = calc_Deff(X2)
    A2eff = calc_Aeff(X2)
    # Calculate center  balance for continous variables (Optimal is at 1)
    colsum = np.sum(Anorm, axis=0)
    centereff = 100 * (1 - np.sum(abs(colsum)) / np.sum(abs(Anorm)))
    # Calculate level balance
    sumdiff = 0.0
    for Ai, nlevel in zip(Array.T, fac_levels):
        Ai = Ai.astype(int)
        nfac = runsize / nlevel
        for j in np.unique(Ai):
            diff = abs(nfac - len(Ai[Ai == j]))
            sumdiff += np.min([abs(diff), nfac])

    # Set to one for perfect balance:
    leveleff = 100 * (1 - sumdiff / np.size(Array))
    # Pearson Correlation
    Acor_pearson = np.corrcoef(Anorm.T)
    # calculate Orthogonality :
    Aortho = np.dot(Anorm.T, Anorm)
    Aortho /= np.diagonal(Aortho)
    orthoeff = 100 * np.linalg.det(Aortho)
    ### Calculate Two-Level Interaction Balance:
    twoleveleff, twolevelmin, Alevel2bal = calc_twofactorbalance(setup, Array)
    twoleveleff, twolevelmin = 100 * twoleveleff, 100 * twolevelmin
    """
    # Calculate canonical correlation
    Acor_can = np.full((number_of_factors, number_of_factors), np.nan)
    try:
        for i, j in itertools.combinations(range(number_of_factors), 2):
            cca = CCA(n_components=1)
            Xnorm_c1, Xnorm_c2 = cca.fit_transform(
                Anorm[:, i].reshape(-1, 1).tolist(), Anorm[:, j].reshape(-1, 1).tolist()
            )
            Acor_can[i, j] = np.corrcoef(Xnorm_c1[:, 0].reshape(-1,1), Xnorm_c2)[0, 1] # seems to be a issue with numpy 1.20
        Acor_can_avg = np.nanmean(abs(Acor_can))
        Acor_can_max = np.nanmax(Acor_can)
    except:
        Acor_can_avg = np.nan
        Acor_can_max = np.nan
    """

    if printopt:
        print("Center Balance : %.2f" % centereff)
        print("Level Balance : %.2f" % leveleff)
        print("Orthogonal Balance: %.2f" % orthoeff)
        print("Two Level interaction Balance: %.2f" % twoleveleff)
        print("Two Level Interaction Minimum One: %.2f" % twolevelmin)
        print("D-efficiency : %.2f" % Deff)
        print("D1-efficiency : %.2f" % D1eff)
        print("D2-efficiency : %.2f" % D2eff)
        print("A-efficiency : %.2f" % Aeff)
        print("A1-efficiency : %.2f" % A1eff)
        print("A2-efficiency : %.2f" % A2eff)
        #print("Average Canonical Corr: %.2f" % Acor_can_avg)
        #print("Maximum Canonical Corr : %.2f" % Acor_can_max)

    # Save output daignostic tables
    if dir_out is not None:
        os.makedirs(dir_out, exist_ok=True)
        # Canonical Correlation
        """
        df_Acor_can = pd.DataFrame(
            data=np.round(Acor_can, 4),
            columns=setup.factor_names,
            index=setup.factor_names,
        )
        df_Acor_can.to_csv(dir_out + "Table_Canonical_Correlation.csv")
        """
        df_pearson = pd.DataFrame(
            data=np.round(Acor_pearson, 4),
            columns=setup.factor_names,
            index=setup.factor_names,
        )
        df_pearson.to_csv(dir_out + "Table_Pearson_Correlation.csv")
        # Alevel2bal.to_csv(dir_out + 'Table_Interaction_Balance.csv')
        with open(dir_out + "Table_Interaction_Balance.txt", "w") as outfile:
            outfile.write("\n".join(str(item) for item in Alevel2bal))
        # with open(dir_out + 'Table_Interaction_Balance.txt', 'w') as f:
        # 	for item in Alevel2bal:
        # 		f.write("%s\n" % item)
        # Make pairwise realtionship plot
        if plotgrid:
            dfarray = pd.DataFrame(Array, columns=setup.factor_names)
            # Define opaque value for scatter points so that one can identify point balance
            # Additional the plot will fit linear regression, which incdicates pariwise orthogonality
            # alphaval = 1/(np.nanmean(Alevel2bal) + 2 * np.nanstd(Alevel2bal) + 1)
            plt.ioff()  # automatic disables display of figures

            def hide_current_axis(*args, **kwds):
                # hide upper triangle
                plt.gca().set_visible(False)

            ax = sns.pairplot(
                dfarray, kind="reg", plot_kws=dict(scatter_kws={"alpha": 0.2})
            )
            ax.map_upper(hide_current_axis)
            plt.savefig(dir_out + "pairwise_correlation.png", dpi=150)
    efficiencies = (
        centereff,
        leveleff,
        orthoeff,
        twoleveleff,
        twolevelmin,
        Deff,
        D1eff,
        D2eff,
        Aeff,
        A1eff,
        A2eff,
        #Acor_can_avg,
        #Acor_can_max,
    )
    return efficiencies


def create_model(Array, mode=2, norm=True, intercept=1):
    # Construct X matrix model from design array
    nrun, nfac = Array.shape
    # Normalize Array
    if norm:
        Array = normalize_array(Array)
    # Define Model Matrix
    # Elements for two- way interaction model
    if mode == 1:
        # Model with intercept and main factor only, no quadratic or interaction terms
        p = nfac + 1
        X = np.zeros((nrun, p))
        # Set intercept to 1
        X[:, 0] = intercept
        X[:, 1:] = Array
        header_main = ["X" + str(i + 1) for i in range(nfac)]
        header = ["Intercept"] + header_main
    elif mode == 2:
        # intercept, main, and main quadratic terms w/o interaction terms
        # Same as JMP calculation
        Array_2way = []  # np.zeros((nrun, nfac * (nfac - 1) / 2))
        header_2way = []
        for i in range(nfac):
            Array_2way.append(Array[:, i] * Array[:, i])
            header_2way.append("X" + str(i + 1) + "X" + str(i + 1))
    elif mode == 3:
        # intercept, main and two-way interaction terms only
        Array_2way = []  # np.zeros((nrun, nfac * (nfac - 1) / 2))
        header_2way = []
        for i in range(nfac - 1):
            for j in range(i + 1, nfac):
                Array_2way.append(Array[:, i] * Array[:, j])
                header_2way.append("X" + str(i + 1) + "X" + str(j + 1))
    elif mode == 4:
        # intercept, main, qadratric and two way interaction terms
        Array_2way = []  # np.zeros((nrun, nfac * (nfac - 1) / 2))
        header_2way = []
        for i in range(nfac):
            for j in range(i, nfac):
                Array_2way.append(Array[:, i] * Array[:, j])
                header_2way.append("X" + str(i + 1) + "X" + str(j + 1))
    if mode > 1:
        Array_2way = np.stack(Array_2way).T
        p = 1 + nfac + Array_2way.shape[1]
        X = np.zeros((nrun, p))
        X[:, 0] = intercept
        X[:, 1 : nfac + 1] = Array
        X[:, nfac + 1 :] = Array_2way
        header_main = ["X" + str(i + 1) for i in range(nfac)]
        header = ["Intercept"] + header_main + header_2way
    return X, header


def calc_Deff(X):
    # Calculation of D-efficiency according to SAS JMP
    XX = np.dot(X.T, X)
    det = np.linalg.det(XX)
    if det > 1e-18:
        det = np.power(det, 1 / X.shape[1])
    else:
        det = 0
    return 100 / X.shape[0] * det


def calc_Aeff(X):
    # Calculation of A-efficiency according to SAS JMP
    XX = np.dot(X.T, X)
    trace = np.trace(np.linalg.pinv(XX))
    aeff = 100 * X.shape[1] / (X.shape[0] * trace)
    if aeff > 100:
        return 100
    else:
        return aeff


def optimize_design(
    setup,
    runsize,
    outpath_nrun=None,
    runtime=100,
    printopt=True,
    nrestarts=10,
    niter=None,
):
    """ 
	Optimizes design for given design specification and  array length (runsize)
	
	This optimization leverages part of the the oapackage.Doptimize package. 
	See for more details https://oapackage.readthedocs.io/en/latest/index.html
	Parameters for oapackage have been finetuned through testing various design setups.
	The oapackage returns multiple designs and the best design is selected based on 
	center balance effciency, orthogonality, and two-level balance (see function evaluate_design2)
	INPUT
	setup: ExperimentalSetup
	runsize: Number of experiments
	outpath_nrun: path for output directory (If None, no files are saved nor plotted)
	runtime: Maximum time for optimization (Default 100 seconds)
	printopt: (Default True) Prints status messages
	nrestart: Number of restrats for optimization (Default 10)
	niter: (Default None) Number of iterations for optimization. If None are given iterations 
	are approximated by runtime 
	"""
    # Setting for oapackage optimisation weighting for D, Ds, D1 efficiencies:
    alpha = [5, 5, 15]
    arrayclass = oapackage.arraydata_t(
        setup.factor_levels, runsize, 0, setup.number_of_factors
    )

    # First estimate time
    if niter is None:
        # First caculate number of iterations for given time
        start_time = time.time()
        devnull = open(os.devnull, "w")
        with redirect_stdout(devnull):
            scores, design_efficiencies, designs, ngenerated = oapackage.Doptimize(
                arrayclass, nrestarts=10, niter=100, optimfunc=alpha
            )
        delta_time1 = time.time() - start_time  # in seconds
        start_time = time.time()
        with redirect_stdout(devnull):
            scores, design_efficiencies, designs, ngenerated = oapackage.Doptimize(
                arrayclass, nrestarts=10, niter=200, optimfunc=alpha
            )
        delta_time2 = time.time() - start_time  # in seconds
        delta_time = delta_time2 - delta_time1
        fac_time = runtime / delta_time
        # print("delta_time: ", delta_time)
        niter = int(100 * fac_time)
        print("Niteration:", niter)
    with redirect_stdout(devnull):
        scores, design_efficiencies, designs, ngenerated = oapackage.Doptimize(
            arrayclass,
            nrestarts=nrestarts,
            niter=niter,
            optimfunc=alpha,
            nabort=3000,
            maxtime=runtime,
        )
    # Evaluate D efficiencies as weighted mean over D, Ds and D1 with weights alpha
    # Deff=[np.sum(d.Defficiencies() * np.asarray(alpha) / np.asarray(alpha).sum()) for d in designs]
    # Make evaluation based on center balance, orthogonality, and two-levelbaldance
    score = []
    for i in range(len(designs)):
        effs = evaluate_design2(setup, np.asarray(designs[i]), printopt=False)
        # score = centereff + orthoeff + 0.5 * twoleveleff
        score.append(effs[0] + effs[2] + 0.5 * effs[3])
    best = np.argmax(score)
    # design_efficiencies[~np.isfinite(design_efficiencies)] = 0.
    # Deffm_list = np.nansum(design_efficiencies * np.asarray(alpha), axis =1) / np.asarray(alpha).sum()
    # best=np.argmax(Deffm_list)
    Asel = designs[best]
    A = np.asarray(Asel)
    # Calculate efficiencies:
    efficiencies = evaluate_design2(
        setup, np.asarray(A), dir_out=outpath_nrun, printopt=True
    )
    # Save results if outpath_nrun is not None
    if outpath_nrun is not None:
        os.makedirs(outpath_nrun, exist_ok=True)
        fname = "EDarray_" + str(setup.factor_levels) + "_Nrun" + str(runsize) + ".csv"
        np.savetxt(os.path.join(outpath_nrun, fname), A, delimiter=",", fmt="%i")
        fname = (
            "Efficiencies_" + str(setup.factor_levels) + "_Nrun" + str(runsize) + ".csv"
        )
        np.savetxt(os.path.join(outpath_nrun, fname), efficiencies, delimiter=",")
    return Asel, efficiencies


def eval_extarray(setup, path, infname):
    """
	Evaluates any given design array (e.g. externally created by SAS) 
	and saves design efficiencies as csv file.
	INPUT:
	setup: ExperimentalSetup
	path, infname: input path and filename of array saved as csv file (without header or index)
	"""
    A = np.loadtxt(os.path.join(path, infname), delimiter=",")
    effs = evaluate_design2(setup, A)
    effs = np.round(effs, 3)
    # centereff, leveleff, orthoeff, twoleveleff, twolevelmin, Deff, D1eff, D2eff, Aeff, A1eff, A2eff = effs
    # Save efficienices as Pandas and csv
    header = [
        "Center Balance",
        "Level Balance",
        "Orthogonality",
        "Two-level Balance",
        "Two-level Min-Eff",
        "D-Eff",
        "D1-Eff",
        "D2-Eff",
        "A-Eff",
        "A1-Eff",
        "A2-Eff",
        #"Canonical-Eff",
        #"Canonical-Corr-max",
    ]
    df_eff = pd.DataFrame(data=effs.reshape(1, -1), columns=header)
    # Assuming infname for input array ends alerady with .csv :
    outfname = "Efficiencies_" + infname
    df_eff.to_csv(os.path.join(path,outfname), index=False)


class ExperimentalSetup:
    def __init__(self, factor_levels, level_values, factor_names):
        self.factor_levels = factor_levels
        self.level_values = level_values
        self.factor_names = factor_names
        self.number_of_factors = len(self.factor_levels)

    @classmethod
    def read(cls, fname_setup):
        return cls(*read_setup_new(fname_setup))


def read_setup(fname_setup):
    """
	Reading in experiment design setup file
	(assume format same as template that is created with create_setupfile.py)
	INPUT
	fname_setup: path + filename for experiment setup excel file
	RETURN
	list of number of factor levels, list of all factor levels
	"""
    df = pd.read_excel(fname_setup)
    nlevel = df["Level Number"].values
    nfactor = len(nlevel)
    factor_names = df["Parameter Name"].values
    factype = df["Parameter Type"].values
    lmin = df["Minimum"].values
    lmax = df["Maximum"].values
    level_vals = []
    for i in range(nfactor):
        if factype[i] == "Categorical":
            level = ["L" + str(k + 1) for k in range(nlevel[i])]
        elif factype[i] == "Continuous":
            if np.isfinite(lmin[i]) & np.isfinite(lmax[i]):
                level = np.linspace(lmin[i], lmax[i], nlevel[i]).tolist()
            else:
                print(
                    "Mimimum or Maximum not specified for Parameter "
                    + str(factor_names[i])
                    + ". Assuming default range from -1 to 1."
                )
        elif factype[i] == "Discrete":
            if np.isfinite(lmin[i]) & np.isfinite(lmax[i]):
                level = np.linspace(lmin[i], lmax[i], nlevel[i], dtype="int8").tolist()
            else:
                print(
                    "Mimimum or Maximum not specified for Parameter "
                    + str(factor_names[i])
                    + ". Assuming default range from -1 to 1."
                )
        else:
            print("Error: Parameter Type unknown for " + str(factor_names[i]))
        level_vals.append(level)
    return nlevel.tolist(), level_vals, factor_names


def read_setup_new(fname_setup):
    """
    Reading in experiment design setup file
    (assume format same as template that is created with create_setupfile.py)
    INPUT
    fname_setup: path + filename for experiment setup excel file
    RETURN
    list of number of factor levels, list of all factor levels
    """
    df = pd.read_excel(fname_setup, na_filter = False)
    if 'Include (Y/N)' in list(df):
        df = df[df['Include (Y/N)'] == 'Yes']
    nlevel = df["Level Number"].values
    if 'Levels' in list(df):
        levels = df["Levels"].values
    else:
        levels = [''] * len(df)
    factor_names = df["Parameter Name"].values
    nfactor = len(factor_names)
    factype = df["Parameter Type"].values
    lmin = df["Minimum"].values
    lmax = df["Maximum"].values
    level_vals = []
    for i in range(nfactor):
        if (levels[i] != '') | (',' in levels[i]):
            print(levels[i])
            level = [ x.strip() for x in levels[i].split(',') ]
            check_numeric = np.asarray([ x.isnumeric() for x in level ])
            if check_numeric.all():
                try:
                    level = np.asarray(level).astype(float).tolist()
                except:
                    factype[i] == "Categorical"
            if len(level) != levels[i]:
                print('WARNING: NUmber of levels not consistent with levels given')
                levels[i] = len(level)
            level_given = True
        else:
            level_given = False
        if not level_given:
            if factype[i] == "Categorical":
                print('WARNING: Level names not given for factor ' + str(factor_names[i]))
                print('Setting level names to L1 ... L' + str(nlevel[i]))
                level = ["L" + str(k + 1) for k in range(nlevel[i])]
            elif factype[i] == "Continuous":
                if np.isfinite(lmin[i]) & np.isfinite(lmax[i]):
                    level = np.linspace(lmin[i], lmax[i], nlevel[i]).tolist()
                else:
                    print(
                        "Mimimum or Maximum not specified for Parameter "
                        + str(factor_names[i])
                        + ". Assuming default range from -1 to 1."
                    )
            elif factype[i] == "Discrete":
                if np.isfinite(lmin[i]) & np.isfinite(lmax[i]):
                    level = np.linspace(lmin[i], lmax[i], nlevel[i], dtype="int8").tolist()
                else:
                    print(
                        "Mimimum or Maximum not specified for Parameter "
                        + str(factor_names[i])
                        + ". Assuming default range from -1 to 1."
                    )
            else:
                print("Error: Parameter Type unknown for " + str(factor_names[i]))
        level_vals.append(level)
    return nlevel.tolist(), level_vals, factor_names


def array2valuetable(setup, fname_array, fname_out):
    """
	Generates experiment design table with level values from optimised experiment array.
	level values and factor names are obtained with ExperimentalSetup
	
	INPUT
	setup: ExperimentalSetup
	fname_array: input path + filename for exp design array in csv format (no header in csv)
	fname_out: output path + filename for converted experimental table with actual level values for each run
	RETURN
	Experiment design table saved as csv file
	"""
    # Read in exp design csv file:
    dfarray = pd.read_csv(fname_array, names=setup.factor_names, header=None)
    dfnew = dfarray.copy()
    # Check again that experiment design aray has the correct number of factors:
    assert dfnew.shape[1] == len(setup.factor_names)
    # replace array levels with actual values
    for i in range(len(setup.factor_names)):
        # loop over each factor
        facname = setup.factor_names[i]
        levels_array = dfarray[facname].unique()
        lvals = setup.level_values[i]
        for nl, level in enumerate(levels_array):
            # loop over each level and replace array level with level value
            dfnew.loc[dfarray[facname] == level, facname] = lvals[nl]
    # Add experiment run number
    dfnew.index += 1
    dfnew.to_csv(fname_out, index_label="Nexp")


def post_evaluate(setup, inpath, outpath, nmin, nmax, ndelta):
    """
	Evaluates design for given range of exp run numbers 
	Returns design efficiency plots and saves arrays as csv in defulat outpath directoy
	INPUT
	setup: ExperimentalSetup
	inpath: input directory name
	nmin: from number of runs to start with
	namx: to number of runs
	ndelta: run n umber interval
	Returns:
	efficiency array
	"""
    # Need updated parameter and efficiencies calculation
    xrun = np.arange(nmin, nmax, ndelta)
    effs_array = np.zeros((len(xrun), 11))
    for i, irun, in enumerate(xrun):
        print("--------------------------------")
        print("Evaluation of array for " + str(irun) + " runs:")
        A = np.loadtxt(
            os.path.join(
                inpath,
                "EDarray_"
                + str(setup.factor_levels)
                + "_Nrun"
                + str(int(irun))
                + ".csv",
            ),
            delimiter=",",
        )
        effs = evaluate_design2(setup, A)
        effs_array[i] = effs
    header = [
        "Center Balance",
        "Level Balance",
        "Orthogonality",
        "Two-way Level Balance",
        "Two-way Min-Eff",
        "D-Eff",
        "D1-Eff",
        "D2-Eff",
        "A-Eff",
        "A1-Eff",
        "A2-Eff",
        #"Canonical-Eff",
        #"Canonical-Corr-max",
    ]
    df_eff = pd.DataFrame(data=effs_array, columns=header)
    dfout = np.round(df_eff, 3)
    dfout["Nexp"] = xrun
    fname = "Efficiencies_" + str(setup.factor_levels) + "_combined.csv"
    dfout.to_csv(os.path.join(outpath, fname), index=False)
    plt.ioff()
    dfout.plot(
        "Nexp",
        [
            "Center Balance",
            "Level Balance",
            "Orthogonality",
            "Two-way Level Balance",
            "Two-way Min-Eff",
            "D-Eff",
            "D1-Eff",
        ],
        sort_columns=True,
    )
    plt.savefig(os.path.join(outpath, "Efficiencies_" + str(setup.factor_levels) + ".png"), dpi=300)
    plt.close()
    return effs_array


def print_designselection_summary(results, fname_out=None):
    """
	print summary evalutaion of three selected designs
	fout: output path + filename. If None prints to sys.stdout (Default)
	"""
    if fname_out is None:
        fout = sys.stdout
        closing = False
    else:
        fout = open(fname_out, "a")
        closing = True

    print("", file=fout)
    print("RESULTS OVERVIEW:", file=fout)
    print("--------------------------------", file=fout)
    for result in results.values():
        print(
            result.name.title() + " Exp Design Runsize: " + str(result.runsize),
            file=fout,
        )
    print("--------------------------------", file=fout)
    print("", file=fout)
    print("", file=fout)
    print("Efficiencies: ", file=fout)
    print(
        "------------------------------------------------------------------------------",
        file=fout,
    )
    columns = [
        result.name.title() + " Design" for result in results.values()
    ]  # ["Min Design", "Opt Design", "Best Desig"]
    names = [
        "Center Balance",
        "Level Balance",
        "Orthogonality",
        "Two-Way Interact Bal",
        "D Efficieny",
        "D1 Efficieny",
    ]
    dfsummary = pd.DataFrame(
        np.round(
            np.array([result.effs[[0, 1, 2, 3, 5, 6]] for result in results.values()]),
            3,
        ).T,
        index=names,
        columns=columns,
    )
    print(dfsummary.to_markdown(), file=fout)
    if closing:
        fout.close()


Result = namedtuple("Result", ["name", "runsize", "effs"])


def main(
    fname_setup,
    path=None,
    outpath=None,
    nrun_max=150,
    maxtime_per_run=100,
    delta_nrun=None,
):
    if outpath is None:
        outpath = path = Path(path)
    else:
        outpath = Path(outpath)

    # Read-in experimental design specifications:
    setup = ExperimentalSetup.read(os.path.join(path,fname_setup))
    # print(setup.level_values)

    # Calculate number of total possible combinations of all factors:
    Ncombinations_total = np.product(np.asarray(setup.factor_levels))
    print("Number of total combinations (Full Factorial):", Ncombinations_total)

    ### Make designs for multiple run sizes:
    maxfact = np.max(setup.factor_levels)
    minfact = np.min(setup.factor_levels)
    # Set run number by factor of lowest common multiple
    ndelta = np.lcm(minfact, maxfact)
    # Find mimimum number of runs:
    nrun, nrun_min = 0, 0
    while nrun_min == 0:
        nrun += ndelta
        if nrun >= setup.number_of_factors + 1:
            nrun_min = nrun

    # Generate optimised design array and calculate efficiencies for each runsize in range of:
    xrun = np.arange(nrun_min, nrun_max, ndelta)
    # Total run time estimate given 100s per run
    minutes = np.round(len(xrun) * 100 / 60, 2)
    # seconds = np.round((60 * (len(xrun) * 100 / 60 - minutes)))
    if minutes < 3:
        print("Hold down your bagel.")
    elif (minutes >= 3) & (minutes < 15):
        print("Perfect time for a cup of tea.")
    elif (minutes >= 15) & (minutes < 180):
        print("Sit back and relax. This may take a while.")
    elif minutes >= 180:
        print("You may want to consider a smaller runsize.")
    print("Total estimated runtime:  " + str(minutes) + "minutes")
    # xrun = np.arange(201,300,ndelta)
    effs_array = np.zeros((len(xrun), 11))
    for i, irun, in enumerate(xrun):
        print("--------------------------------")
        print("Optimizing array for " + str(irun) + " runs ...")
        outpath_nrun = os.path.join(outpath, "DesignArray_Nrun" + str(int(irun)) + "/")
        Aopt, effs = optimize_design(
            setup, int(irun), runtime=maxtime_per_run, outpath_nrun=outpath_nrun
        )
        effs_array[i] = effs

    print("")
    print("-------------------------------------------")
    print("Finished optimising all possible run sizes.")

    print("Saving efficiencies and design arrays as csv files....")
    # Save efficienices as Pandas and csv
    # header = ['Center Balance', 'Level Balance', 'Orthogonality', 'Two-level Balance', 'Two-level Min-Eff',  'avgD-Eff', 'D-Eff', 'Ds-Eff', 'D1-Eff', 'A-Eff', 'E-Eff']
    header = [
        "Center Balance",
        "Level Balance",
        "Orthogonality",
        "Two-level Balance",
        "Two-level Min-Eff",
        "D-Eff",
        "D1-Eff",
        "D2-Eff",
        "A-Eff",
        "A1-Eff",
        "A2-Eff",
        #"Canonical-Eff",
        #"Canonical-Corr-max",
    ]
    df_eff = pd.DataFrame(data=effs_array, columns=header)
    dfout = np.round(df_eff, 3)
    dfout["Nexp"] = xrun
    fname = "Efficiencies_" + str(setup.factor_levels) + "_all3.csv"
    dfout.to_csv(os.path.join(outpath, fname), index=False)

    # dfout.plot("Nexp", ['Center Balance', 'Level Balance', 'Orthogonality', 'D-Eff', 'D1-Eff', 'A-Eff', 'E-Eff'], sort_columns = True)
    dfout.plot(
        "Nexp",
        [
            "Center Balance",
            "Level Balance",
            "Orthogonality",
            "Two-level Balance",
            "Two-level Min-Eff",
            "D1-Eff",
        ],
        sort_columns=True,
    )
    plt.ioff()
    plt.savefig(os.path.join(outpath, "Efficiencies_" + str(setup.factor_levels) + ".png"), dpi=300)
    plt.close()
    # seems to be correlation  between orthogonality(levelbvalance) and D1 and
    # plt.scatter(effs_array[:,1], effs_array[:,5])

    ###### Identify minimum, optimal, and best runsize for experiment:
    results = {}
    """Minimum run criteria:
	runsize > Nfactor + 1
	center balance > 95
	level balance > 95
	Orthogonal Balance > 90 
	Two Level interaction Balance > 90
	Two Level Interaction Minimum One = 100
	"""
    print("Finding minimum, optimal and best designs...")
    effs_minsel = np.where(
        (effs_array[:, 0] >= 95)
        & (effs_array[:, 1] >= 95)
        & (effs_array[:, 2] >= 90)
        & (effs_array[:, 3] >= 90)
        & (effs_array[:, 4] == 100)
    )[0]
    if len(effs_minsel) > 0:
        ix_minsel = effs_minsel[0]
        results["min"] = Result("minimum", xrun[ix_minsel], effs_array[ix_minsel])

    """Optimal run criteria:
	center balance > 98
	level balance > 98
	Orthogonal Balance > 95 
	Two Level interaction Balance > 95
	Two Level Interaction Minimum One = 100
	Best score = center balance + Orthogonal Balance + Two Level interaction Balance + mean(D eff, D1 eff) - 4/run_min * nruns
	The penalty score is 4percent/minimum_runsize per additional run. 
	That means every time an additional minimum runsize is added, the score must be at least by 1 percent better
	on the average efficiency (center, orthogonality, two-way level balance, D efficiency)
	"""
    effs_optsel = np.where(
        (effs_array[:, 0] >= 98)
        & (effs_array[:, 1] >= 98)
        & (effs_array[:, 2] >= 95)
        & (effs_array[:, 3] >= 95)
        & (effs_array[:, 4] == 100)
    )[0]
    runs_sel = xrun[effs_optsel]
    if len(effs_optsel) > 0:
        score = (
            effs_array[effs_optsel, 0]
            + effs_array[effs_optsel, 2]
            + effs_array[effs_optsel, 3]
            + 0.5 * effs_array[effs_optsel, 5]
            + 0.5 * effs_array[effs_optsel, 6]
            - 4 / results["min"].runsize * runs_sel
        )
        irun_optsel = np.argmax(score)
        run_optsel = runs_sel[irun_optsel]
        ixopt = np.where(xrun == run_optsel)
        results["opt"] = Result("optimal", run_optsel, effs_array[ixopt].flatten())

    """
	Best run:
	score based on sum of effcieciencies and includes a small penalty for runsize relative to maximum runsize
	"""
    bestscore = (
        effs_array[:, 0]
        + effs_array[:, 2]
        + effs_array[:, 3]
        + 100 * (effs_array[:, 4] - 100)
        + 0.5 * effs_array[:, 5]
        + 0.5 * effs_array[:, 6]
        - 1 / nrun_max * nrun
    )
    irun_bestsel = np.argmax(bestscore)
    run_bestsel = xrun[irun_bestsel]
    ixbest = np.where(xrun == run_bestsel)
    results["best"] = Result("best", run_bestsel, effs_array[ixbest].flatten())

    # Convert exp arrays into design tables with user level values
    print("Saving minimum, optimal, and best design as experiment design tables...")
    for result in results.values():
        fname_array = (os.path.join(outpath,
            "DesignArray_Nrun"
            + str(result.runsize)
            + "/"
            + "EDarray_"
            + str(setup.factor_levels)
            + "_Nrun"
            + str(result.runsize)
            + ".csv")
        )
        fname_out = (os.path.join(outpath,
            "Designtable_"
            + result.name
            + "_Nrun"
            + str(result.runsize)
            + ".csv")
        )
        array2valuetable(setup, fname_array, fname_out)

    # print summary in terminal:
    print_designselection_summary(results)
    # and in addition print summary to file:
    print_designselection_summary(
        results, fname_out=os.path.join(outpath, "Experiment_Design_selection_summary.txt")
    )
    print("")
    print("FINISHED")


### Possible add-on later: subsequent longer optimisation for the three final designs: min, opt, and best


def main_cli():
    ap = argparse.ArgumentParser()
    ap.add_argument("settings_path", nargs="?", default="settings_design.yaml")
    args = ap.parse_args()
    print(f"using settings in: {args.settings_path!r}")
    with open(args.settings_path) as f:
        cfg = yaml.safe_load(f)
    main(**cfg)


if __name__ == "__main__":
    main_cli()



from setuptools import setup, find_packages
from numpy.distutils.core import setup, Extension
from os import path
import os
import subprocess
import io

## in development set version
PYPI_VERSION = '0.4.1'

# Return the git revision as a string (from numpy)
def git_version():
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(cmd, stdout = subprocess.PIPE, env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', '--short', 'HEAD'])
        GIT_REVISION = out.strip().decode('ascii')
    except OSError:
        GIT_REVISION = "Unknown"

    return GIT_REVISION


if PYPI_VERSION is None:
    PYPI_VERSION = git_version()


this_directory = path.abspath(path.dirname(__file__))
with io.open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

packages = find_packages()

if __name__ == "__main__":
    setup(name = 'DoEgen',
          author            = "Sebastian Haan",
          author_email      = "sebastian.haan@sydney.edu.au",
          url               = "https://github.com/sebhaan/DoEgen",
          version           = PYPI_VERSION,
          description       = "DoEgen: A Python Library for Optimised Design of Experiment Generation and Evaluation",
          long_description  = long_description,
          long_description_content_type='text/markdown',
          install_requires  = ['numpy>=1.16.3', 
                              'xlrd>=1.2.0',
                              'pandas>=1.0.3',
                              'XlsxWriter>=1.2.8',
                              'seaborn>=0.11.1',
                              'OApackage==2.6.6',
                              'tabulate==0.8.7',
                              'matplotlib>=3.1.0',
                              'PyYAML>=5.3.1',
                              'scikit_learn>=0.22.2.post1'],
          python_requires   = '>=3.6',
          setup_requires    = ["pytest-runner", 'webdav'],
          tests_require     = ["pytest", 'webdav'],
          packages          = ['doegen'],
          package_data      = {'doegen': ['*.yaml',
                                          '*.xlsx',
                                        'test/Experiment_setup_test.xlsx',
                                        'test/settings_design_test.yaml',
                                        'test/settings_expresults_test.yaml',
                                        'test/results/experiment_results_Nrun72.xlsx',
                                        'test/results/Designtable_optimal_Nrun72.csv']},
          include_package_data = False,
          classifiers       = ['Programming Language :: Python :: 3',
                               'Programming Language :: Python :: 3.6',
                               'Programming Language :: Python :: 3.7'
                               ]
          )


"""
Run this initialisation of .yaml and .xlsx files after installation of doegen.

Creates config yaml files and excel templates if not already existing in current working directory.
"""
import os
import sys
import shutil
import doegen

#Current working directory
cwd = os.getcwd()

#Directory where package doegen is installed:
pckd = doegen.__path__[0] 

# Create settings_design.yaml
if os.path.exists('test'):
	print("directory already exist: test")
else:
	source = os.path.join(pckd,'test')
	target = os.path.join(cwd,'test')
	try:
		shutil.copytree(source, target)
		print("Generated directory: test")
	except IOError as e:
		print("Unable to copy directory. %s" % e)
	except:
		print("Unexpected error:", sys.exc_info())